!   Name: read_qcsw_daily.f90
!   Purpose:  This code reads the daily average fields of surface shortwave fluxes
!   of the quality-check SW (QCSW) algorithm.  The code then converts the variables 
!   to true 1x1 degree fields.
!
!   Variables:
!   Name      Type                 Description
!   sw_var    real(44016)          field of file, read one at a time, in the following 
!                                  order:
!                                    (1)      Clear-sky surface insolation  (FCLR)
!                                    (2)      All-sky surface insolation  (FALL)
!                                    (3)      Surface absorbed SW flux  (FABS)
!                                    (4)      All-sky Surface Albedo  (SALB)
!   var0      real (44016)         Input field on the nested grid
!   var1      real (64800)         Intermediate 1x1 field, 1-dimensional array
!   var2      real (360,180)       Output field on 1x1 grid, 2-dimensional array
!
!   The layout of the nested grid is described in the readme files.
!   both, input (nested) and output 1x1 grids have the same orientation:
!   start at the greenwich meridian-south pole and go east and north.
!
!   Internal Routines:
!   Subroutine Regrid- Regrids a field from the nested grid to an equal-angle 1x1 grid
!   Subroutine Unnest- Replicates field values for the Regrid routine.
!   Subroutine native_4byte_real- Converts byte order from big to little endian
!
!   Input files:
!   qcsw_daily.nml - namelist file to select year, month, input & output
!                      paths and which variables to extract
!   srb_rel2.5_qcsw_daily_YYYMM.binary -data file where YYYY is 4-digit year and
!                      MM is 2-digit month.  This will be filled by the program.
!
!   Output files (if variable is chosen in namelist):
!     set ascii and/or binary logical variable in namelist file to true for output
!   FCLR_daily_YYYMM.ascii   (binary)
!   FALL_daily_YYYMM.ascii   (binary)
!   FABS_daily_YYYMM.ascii   (binary)
!   SALB_daily_YYYMM.ascii   (binary)


       program read_qcsw_daily

       implicit none

       integer, parameter :: rkind=selected_real_kind(6), ikind=selected_int_kind(6)
       integer(ikind), parameter :: nreg=44016, sreg=64800, nlon=360, nlat=180, nvars=4
       integer(ikind), dimension(12) :: mdays = (/31,28,31,30,31,30,        &
                                          31,31,30,31,30,31/)

       character (len=200) :: infile, outfile, outfile2, path_in, path_out
       character (len=7), dimension (nvars) :: var_header
       character(len=4) :: cyear
       character(len=2) :: cmon

       integer(ikind) :: yr, mon, iflx , reg
       real (rkind) :: sw_var(nreg)

       integer(ikind) :: nrg, srg, lon, lat, breg, ereg, irec, day
       real (rkind) :: var0(nreg), var1(sreg), var2(nlon,nlat)

       logical :: FCLR, FALL, FABS, SALB
       logical :: var_list(nvars), ascii, binary, little_endian

       namelist/time_vars/yr, mon, ascii, binary, path_in, path_out, little_endian, &
              FCLR, FALL, FABS, SALB

       var_header=(/'FCLR_  ', 'FALL_  ', 'FABS_  ', 'SALB_  '/)

       print*, '*****************************************************************'
       print*, '*                                                               *'
       print*, '*                                                               *'
       print*, '* Data Set srb_rel2.5_qcsw_daily Read Software                  *'
       print*, '*                                                               *'
       print*, '* Version: 1.0                                                  *'
       print*, '*                                                               *'
       print*, '* Contact: Atmospheric Sciences Data Center                     *'
       print*, '* Science, Users and Data Services Office                       *'
       print*, '* Mail Stop 157D                                                *'
       print*, '* 2 South Wright Street                                         *'
       print*, '* NASA Langley Research Center                                  *'
       print*, '* Hampton, Virginia 23681-2199                                  *'
       print*, '* U.S.A.                                                        *'
       print*, '*                                                               *'
       print*, '* E-mail: larc@eos.nasa.gov                                     *'
       print*, '* Phone: (757)864-8656                                          *'
       print*, '* FAX: (757)864-8807                                            *'
       print*, '*                                                               *'
       print*, '*****************************************************************'

!   Open and read namelist file
       open (1,file='qcsw_daily.nml')
       read (1,nml=time_vars)
       close (1)

       write (cyear,"(I4)") yr
       write (cmon,"(I2.2)") mon

!   Leap year check
       if (mon.eq.2.and.mod(yr,4).eq.0)  mdays(mon)=mdays(mon)+1
 
!   Re-assign logicals from namelist
       var_list(1)=FCLR
       var_list(2)=FALL
       var_list(3)=FABS
       var_list(4)=SALB

!   Open input file
       infile=trim(path_in)//'srb_rel2.5_qcsw_daily_'//cyear//cmon//'.binary'
       open (10,file=infile,form='unformatted', access='direct', recl=nreg*4, status='old')
       print*, trim(infile), ' input file is opened'

!   Read and convert desired variables to 1x1 degree global field
       do iflx=1,nvars
          if (var_list(iflx)) then

!   Open binary output file
            if (binary) then
             outfile=trim(path_out)//trim(var_header(iflx))//&
                'daily_'//cyear//cmon//'.binary'
             open (15,file=outfile, form='unformatted', access='direct', &
                  recl=nlon*nlat*4, status='replace')
            end if

!   Open ascii output file
            if (ascii) then
             outfile2=trim(path_out)//trim(var_header(iflx))//&
                'daily_'//cyear//cmon//'.ascii'
             open (25,file=outfile2, form='formatted', status='replace')
            end if

             do day=1,mdays(mon)

                irec=(day-1)*4
                read (10,rec=irec+iflx) sw_var

!   Convert from big endian to little endian byte order, if necessary.
                   if (little_endian) then
                      do reg=1,nreg
                         call native_4byte_real(sw_var(reg),sw_var(reg))
                      end do
                   end if

                var0(:)=sw_var
                call regrid

!   Sample to screen:
                if (day==14 ) then
                   write (*,*) 'Variable ', trim(var_header(iflx)), 'Day = ', day
                   write (*,"(A,4x,5I10)") 'lon # = ', (lon,lon=100,104)
                   do lat=45,51
                      write (*,"(A,I3,5f10.3)") 'lat band # ',lat, &
                               (var2(lon,lat),lon=100,104)
                   end do
                end if


!   Write output files, one field per file
!         binary
                if (binary) write (15,rec=day) var2

!          ascii output, starting at South Pole, Greenwich
               if (ascii) then
                do lat=1,nlat
                   write (25,"(15f10.3)") (var2(lon,lat),lon=1,nlon)
                end do
               end if

             end do  !day
             if (binary) print*, 'file ', trim(outfile), ' has been written'
             if (ascii) print*, 'file ', trim(outfile2), ' has been written'
             if (binary) close (15)
             if (ascii) close (25)
          end if
       end do

!   Close input file
       close (10)
 
       contains
!!!!****************************************************************!!!!
!
      subroutine regrid
!
!!!!****************************************************************!!!!
!
!   this routine is designed to regrid a field available on the nested
!   grid to an equal-angle 1x1 grid.  in this scheme, the larger
!   boxes (in longitude) are simply replicated into the 1x1 boxes
!   contained within them.
!
!!!!****************************************************************!!!!
!
      implicit none
!
      ereg=0
!
!   go through the boxes of nested grid, one at a time, and call
!   subroutine unnest to replicate the value of this box into an
!   appropriate number of 1x1 boxes (depending on the size of the
!   nested grid box).
!
      do nrg=1,nreg
!
        if (nrg.le.3) then
          call unnest(120)
        elseif (nrg.le.408) then
          call unnest(8)
        elseif (nrg.le.1308) then
          call unnest(4)
        elseif (nrg.le.5808) then
          call unnest(2)
        elseif (nrg.le.38208) then
          call unnest(1)
        elseif (nrg.le.42708) then
          call unnest(2)
        elseif (nrg.le.43608) then
          call unnest(4)
        elseif (nrg.le.44013) then
          call unnest(8)
        elseif (nrg.le.44016) then
          call unnest(120)
        end if
!
      end do  !  nrg
!
!   construct a 2-dimensional array from the 1-dimensional array
!   of the 1x1 field.
!
      do srg=1,sreg
!
        lat=(srg-1)/360+1
        lon=srg-(lat-1)*360
!
        var2(lon,lat)=var1(srg)
!
      end do  !  srg
!
      end subroutine regrid
!
!!!!****************************************************************!!!!
!
      subroutine unnest(mreg)
!
!!!!****************************************************************!!!!
!
      implicit none
!
      integer, intent(in) :: mreg
!
      breg=ereg+1
      ereg=breg+mreg-1
!
!   replicate the value from one box of the nested grid into all
!   1x1 boxes contained in it.
!
      do srg=breg,ereg
        var1(srg)=var0(nrg)
      end do  !  nrg
!
      end subroutine unnest
!
!!!!****************************************************************!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!           FILE: SUBR_native_4byte_real.f90
!     SUBPROGRAM: native_4byte_real
!
!         AUTHOR: David Stepaniak, NCAR/CGD/CAS
! DATE INITIATED: 29 April 2003
!  LAST MODIFIED: 29 April 2003
!
!       SYNOPSIS: Converts a 32 bit, 4 byte, REAL from big Endian to
!                 little Endian, or conversely from little Endian to big
!                 Endian.
!
!    DESCRIPTION: This subprogram allows one to convert a 32 bit, 4 byte,
!                 REAL data element that was generated with, say, a big
!                 Endian processor (e.g. Sun/sparc, SGI/R10000, etc.) to its
!                 equivalent little Endian representation for use on little
!                 Endian processors (e.g. PC/Pentium running Linux). The
!                 converse, little Endian to big Endian, also holds.
!                 This conversion is accomplished by writing the 32 bits of
!                 the REAL data element into a generic 32 bit INTEGER space
!                 with the TRANSFER intrinsic, reordering the 4 bytes with
!                 the MVBITS intrinsic, and writing the reordered bytes into
!                 a new 32 bit REAL data element, again with the TRANSFER
!                 intrinsic. The following schematic illustrates the
!                 reordering process
!
!
!                  --------    --------    --------    --------
!                 |    D   |  |    C   |  |    B   |  |    A   |  4 Bytes
!                  --------    --------    --------    --------
!                                                             |
!                                                              -> 1 bit
!                                       ||
!                                     MVBITS
!                                       ||
!                                       \/
!
!                  --------    --------    --------    --------
!                 |    A   |  |    B   |  |    C   |  |    D   |  4 Bytes
!                  --------    --------    --------    --------
!                         |           |           |           |
!                         24          16          8           0   <- bit
!                                                                 position
!
!          INPUT: realIn,  a single 32 bit, 4 byte REAL data element.
!         OUTPUT: realOut, a single 32 bit, 4 byte REAL data element, with
!                 reverse byte order to that of realIn.
!    RESTRICTION: It is assumed that the default REAL data element is
!                 32 bits / 4 bytes.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  SUBROUTINE native_4byte_real( realIn, realOut )
  IMPLICIT NONE
  REAL, INTENT(IN)                              :: realIn
                                                   ! a single 32 bit, 4 byte
                                                   ! REAL data element
  REAL, INTENT(OUT)                             :: realOut
                                                   ! a single 32 bit, 4 byte
                                                   ! REAL data element, with
                                                   ! reverse byte order to
                                                   ! that of realIn
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Local variables (generic 32 bit INTEGER spaces):

  INTEGER                                       :: i_element
  INTEGER                                       :: i_element_br
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Transfer 32 bit of realIn to generic 32 bit INTEGER space:
  i_element = TRANSFER( realIn, 0 )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Reverse order of 4 bytes in 32 bit INTEGER space:
  CALL MVBITS( i_element, 24, 8, i_element_br, 0  )
  CALL MVBITS( i_element, 16, 8, i_element_br, 8  )
  CALL MVBITS( i_element,  8, 8, i_element_br, 16 )
  CALL MVBITS( i_element,  0, 8, i_element_br, 24 )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Transfer reversed order bytes to 32 bit REAL space (realOut):
  realOut = TRANSFER( i_element_br, 0.0 )
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  END SUBROUTINE
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


       end program read_qcsw_daily
